@@ -100,16 +100,18 @@ static jmp_buf	restart;
 
 char tag[TAGLEN];
 static int tagnum;
-#define GENSYM	(sprintf(tag, "a%04d", ++tagnum % TAGMOD), tag)
+#define GENSYM	(sprintf(tag, "A%04d", ++tagnum % TAGMOD), tag)
 
-static char *shroud;	/* string to shroud in debug output, if  non-NULL */
-static int mytimeout;	/* value of nonreponse timeout */
-static int msglen;	/* actual message length */
+static char shroud[PASSWORDLEN];	/* string to shroud in debug output */
+static int mytimeout;			/* value of nonreponse timeout */
+static int msglen;			/* actual message length */
+static int mimemsg;			/* bitmask indicating MIME body-type */
 
 /* use these to track what was happening when the nonresponse timer fired */
-#define GENERAL_WAIT	0
-#define SERVER_WAIT	1
-#define FORWARDING_WAIT	2
+#define GENERAL_WAIT	0	/* unknown wait type */
+#define SERVER_WAIT	1	/* waiting for mailserver response */
+#define LISTENER_WAIT	2	/* waiting for listener initialization */
+#define FORWARDING_WAIT	3	/* waiting for listener response */
 static phase;
 
 static void set_timeout(int timeleft)
@@ -244,7 +246,7 @@ static int is_host_alias(const char *name, struct query *ctl)
     }
 
     /* add this name to relevant server's `also known as' list */
-    save_str(&lead_server->akalist, -1, name);
+    save_str(&lead_server->akalist, name, 0);
     return(TRUE);
 #endif /* HAVE_RES_SEARCH */
 }
@@ -277,7 +279,7 @@ struct idlist **xmit_names;	/* list of recipient names parsed out */
 	}
 	if (outlevel == O_VERBOSE)
 	    error(0, 0, "mapped %s to local %s", name, lname+off);
-	save_str(xmit_names, XMIT_ACCEPT, lname+off);
+	save_str(xmit_names, lname+off, XMIT_ACCEPT);
 	accept_count++;
     }
 }
@@ -319,7 +321,7 @@ struct idlist **xmit_names;	/* list of recipient names parsed out */
 			if (outlevel == O_VERBOSE)
 			    error(0, 0, "passed through %s matching %s", 
 				  cp, idp->id);
-			save_str(xmit_names, XMIT_ACCEPT, cp);
+			save_str(xmit_names, cp, XMIT_ACCEPT);
 			accept_count++;
 			break;
 		    }
@@ -335,7 +337,7 @@ struct idlist **xmit_names;	/* list of recipient names parsed out */
 		 */
 		if (!is_host_alias(atsign+1, ctl))
 		{
-		    save_str(xmit_names, XMIT_REJECT, cp);
+		    save_str(xmit_names, cp, XMIT_REJECT);
 		    reject_count++;
 		    continue;
 		}
@@ -463,6 +465,7 @@ static int smtp_open(struct query *ctl)
 	 */
 	struct idlist	*idp;
 	char *id_me = use_invisible ? ctl->server.truename : fetchmailhost;
+	int oldphase = phase;
 
 	errno = 0;
 
@@ -471,6 +474,10 @@ static int smtp_open(struct query *ctl)
 	 * Use both explicit hunt entries (value TRUE) and implicit 
 	 * (default) ones (value FALSE).
 	 */
+	oldphase = phase;
+	phase = LISTENER_WAIT;
+
+	set_timeout(ctl->server.timeout);
 	for (idp = ctl->smtphunt; idp; idp = idp->next)
 	{
 	    char	*cp, *parsed_host = alloca(strlen(idp->id) + 1);
@@ -520,6 +527,8 @@ static int smtp_open(struct query *ctl)
 	    close(ctl->smtp_socket);
 	    ctl->smtp_socket = -1;
 	}
+	set_timeout(0);
+	phase = oldphase;
     }
 
     if (outlevel >= O_VERBOSE && ctl->smtp_socket != -1)
@@ -628,7 +637,7 @@ int num;		/* index of message */
     char		return_path[HOSTLEN + USERNAMELEN + 4]; 
     int			from_offs, reply_to_offs, resent_from_offs;
     int			app_from_offs, sender_offs, resent_sender_offs;
-    int			ctt_offs, env_offs;
+    int			env_offs;
     char		*headers, *received_for, *destaddr, *rcv, *cp;
     int 		n, linelen, oldlen, ch, remaining, skipcount;
     struct idlist 	*idp, *xmit_names;
@@ -645,10 +654,11 @@ int num;		/* index of message */
     /* read message headers */
     headers = received_for = NULL;
     from_offs = reply_to_offs = resent_from_offs = app_from_offs = 
-	sender_offs = resent_sender_offs = ctt_offs = env_offs = -1;
+	sender_offs = resent_sender_offs = env_offs = -1;
     oldlen = 0;
     msglen = 0;
     skipcount = 0;
+    mimemsg = 0;
 
     for (remaining = fetchlen; remaining > 0 || protocol->delimited; remaining -= linelen)
     {
@@ -658,11 +668,14 @@ int num;		/* index of message */
 	linelen = 0;
 	line[0] = '\0';
 	do {
+	    set_timeout(ctl->server.timeout);
 	    if ((n = SockRead(sock, buf, sizeof(buf)-1)) == -1) {
+		set_timeout(0);
 		free(line);
 		free(headers);
 		return(PS_SOCKET);
 	    }
+	    set_timeout(0);
 	    linelen += n;
 	    msglen += n;
 
@@ -678,7 +691,6 @@ int num;		/* index of message */
 		}
 	    }
 
-	    set_timeout(ctl->server.timeout);
 
 	    line = (char *) realloc(line, strlen(line) + strlen(buf) +1);
 
@@ -817,10 +829,13 @@ int num;		/* index of message */
 	 * If the SMTP server conforms to the standards, and fetchmail gets the
 	 * envelope sender from the Return-Path, the new Return-Path should be
 	 * exactly the same as the original one.
+	 *
+	 * Empty Return-Path headers will be ignored.
+	 *
 	 */
-	if (!strncasecmp("Return-Path:", line, 12))
+	if (!strncasecmp("Return-Path:", line, 12) && (cp = nxtaddr(line)))
 	{
-	    strcpy(return_path, nxtaddr(line));
+	    strcpy(return_path, cp);
 	    if (!ctl->mda) {
 		free(line);
 		continue;
@@ -864,18 +879,19 @@ int num;		/* index of message */
 	else if (!strncasecmp("Resent_Sender:", line, 14))
 	    resent_sender_offs = (line - headers);
 
-	else if (!strncasecmp("Content-Transfer-Encoding:", line, 26))
-	    ctt_offs = (line - headers);
- 	else if (!strncasecmp("Message-Id:", buf, 11 ))
+ 	else if (!strncasecmp("Message-Id:", buf, 11))
 	{
-	    if( ctl->server.uidl )
+	    if (ctl->server.uidl)
  	    {
 	        char id[IDLEN+1];
-	        /* prevent stack overflows */
-		buf[IDLEN+12] = 0;
- 		sscanf( buf+12, "%s", id);
- 	        if( !str_find( &ctl->newsaved, num ) )
- 		    save_str(&ctl->newsaved, num, id );
+
+		buf[IDLEN+12] = 0;		/* prevent stack overflow */
+ 		sscanf(buf+12, "%s", id);
+ 	        if (!str_find( &ctl->newsaved, num))
+		{
+ 		    struct idlist *new = save_str(&ctl->newsaved,id,UID_SEEN);
+		    new->val.status.num = num;
+		}
  	    }
  	}
 
@@ -963,6 +979,16 @@ int num;		/* index of message */
      * In fact we have to, as this will tell us where to forward to.
      */
 
+    /* Decode MIME encoded headers. We MUST do this before
+     * looking at the Content-Type / Content-Transfer-Encoding
+     * headers (RFC 2046).
+     */
+    if (ctl->mimedecode) {
+	UnMimeHeader(headers);
+    }
+    /* Check for MIME headers indicating possible 8-bit data */
+    mimemsg = MimeBodyType(headers);
+
     /*
      * If there is a Return-Path address on the message, this was
      * almost certainly the MAIL FROM address given the originating
@@ -1037,7 +1063,7 @@ int num;		/* index of message */
 	if (!accept_count)
 	{
 	    no_local_matches = TRUE;
-	    save_str(&xmit_names, XMIT_ACCEPT, user);
+	    save_str(&xmit_names, user, XMIT_ACCEPT);
 	    if (outlevel == O_VERBOSE)
 		error(0, 0, 
 		      "no local matches, forwarding to %s",
@@ -1045,7 +1071,7 @@ int num;		/* index of message */
 	}
     }
     else	/* it's a single-drop box, use first localname */
-	save_str(&xmit_names, XMIT_ACCEPT, ctl->localnames->id);
+	save_str(&xmit_names, ctl->localnames->id, XMIT_ACCEPT);
 
 
     /*
@@ -1065,7 +1091,7 @@ int num;		/* index of message */
 	char	*names, *before, *after;
 
 	for (idp = xmit_names; idp; idp = idp->next)
-	    if (idp->val.num == XMIT_ACCEPT)
+	    if (idp->val.status.mark == XMIT_ACCEPT)
 		good_addresses++;
 
 	destaddr = "localhost";
@@ -1092,13 +1118,13 @@ int num;		/* index of message */
 	     * long lists of users and (re)implement %s.
 	     */
 	    for (idp = xmit_names; idp; idp = idp->next)
-		if (idp->val.num == XMIT_ACCEPT)
+		if (idp->val.status.mark == XMIT_ACCEPT)
 		    length += (strlen(idp->id) + 1);
 
 	    names = (char *)xmalloc(++length);
 	    names[0] = '\0';
 	    for (idp = xmit_names; idp; idp = idp->next)
-		if (idp->val.num == XMIT_ACCEPT)
+		if (idp->val.status.mark == XMIT_ACCEPT)
 		{
 		    strcat(names, idp->id);
 		    strcat(names, " ");
@@ -1178,7 +1204,7 @@ int num;		/* index of message */
     }
     else
     {
-	char	*ap, *ctt, options[MSGBUFSIZE], addr[128];
+	char	*ap, options[MSGBUFSIZE], addr[128];
 
 	/* build a connection to the SMTP listener */
 	if ((smtp_open(ctl) == -1))
@@ -1191,22 +1217,16 @@ int num;		/* index of message */
 	}
 
 	/*
-	 * Compute ESMTP options.  It's a kluge to use nxtaddr()
-	 * here because the contents of the Content-Transfer-Encoding
-	 * headers isn't semantically an address.  But it has the
-	 * desired tokenizing effect.
+	 * Compute ESMTP options.
 	 */
 	options[0] = '\0';
-	if (ctl->server.esmtp_options & ESMTP_8BITMIME)
-	    if (ctl->pass8bits)
+	if (ctl->server.esmtp_options & ESMTP_8BITMIME) {
+             if (ctl->pass8bits || (mimemsg & MSG_IS_8BIT))
 		strcpy(options, " BODY=8BITMIME");
-	    else if ((ctt_offs >= 0) && (ctt = nxtaddr(headers + ctt_offs)))
-	    {
-		if (!strcasecmp(ctt,"7BIT"))
-		    strcpy(options, " BODY=7BIT");
-		else if (!strcasecmp(ctt,"8BIT"))
-		    strcpy(options, " BODY=8BITMIME");
-	    }
+             else if (mimemsg & MSG_IS_7BIT)
+		strcpy(options, " BODY=7BIT");
+        }
+
 	if ((ctl->server.esmtp_options & ESMTP_SIZE) && reallen > 0)
 	    sprintf(options + strlen(options), " SIZE=%ld", reallen);
 
@@ -1239,12 +1259,13 @@ int num;		/* index of message */
 		 * coming from this address, probably due to an
 		 * anti-spam domain exclusion.  Respect this.  Don't
 		 * try to ship the message, and don't prevent it from
-		 * being deleted.  Typical values.
+		 * being deleted.  Typical values:
 		 *
 		 * 571 = sendmail's "unsolicited email refused"
 		 * 501 = exim's old antispam response
 		 * 550 = exim's new antispam response (temporary)
 		 */
+		SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
 		free(headers);
 		free_str_list(&xmit_names);
 		return(PS_REFUSED);
@@ -1273,6 +1294,7 @@ int num;		/* index of message */
 		return(PS_TRANSIENT);
 
 	    case 552: /* message exceeds fixed maximum message size */
+	    case 553: /* invalid sending domain */
 		/*
 		 * Permanent no-go condition on the
 		 * ESMTP server.  Don't try to ship the message, 
@@ -1305,7 +1327,7 @@ int num;		/* index of message */
 	destaddr = ctl->smtpaddress ? ctl->smtpaddress : ( ctl->smtphost ? ctl->smtphost : "localhost");
 	
 	for (idp = xmit_names; idp; idp = idp->next)
-	    if (idp->val.num == XMIT_ACCEPT)
+	    if (idp->val.status.mark == XMIT_ACCEPT)
 	    {
 		if (strchr(idp->id, '@'))
 		    strcpy(addr, idp->id);
@@ -1321,7 +1343,7 @@ int num;		/* index of message */
 		else
 		{
 		    bad_addresses++;
-		    idp->val.num = XMIT_ANTISPAM;
+		    idp->val.status.mark = XMIT_ANTISPAM;
 		    error(0, 0, 
 			  "SMTP listener doesn't like recipient address `%s'",
 			  addr);
@@ -1393,7 +1415,7 @@ int num;		/* index of message */
 		else if (good_addresses == 1)
 		{
 		    for (idp = xmit_names; idp; idp = idp->next)
-			if (idp->val.num == XMIT_ACCEPT)
+			if (idp->val.status.mark == XMIT_ACCEPT)
 			    break;	/* only report first address */
 		    if (strchr(idp->id, '@'))
 			sprintf(buf+1, "for <%s>", idp->id);
@@ -1462,7 +1484,7 @@ int num;		/* index of message */
 	    else
 	    {
 		for (idp = xmit_names; idp; idp = idp->next)
-		    if (idp->val.num == XMIT_REJECT)
+		    if (idp->val.status.mark == XMIT_REJECT)
 			break;
 		sprintf(errhd+strlen(errhd), "recipient address %s didn't match any local name", idp->id);
 	    }
@@ -1482,13 +1504,13 @@ int num;		/* index of message */
 	    strcat(errhd, "SMTP listener rejected local recipient addresses: ");
 	    errlen = strlen(errhd);
 	    for (idp = xmit_names; idp; idp = idp->next)
-		if (idp->val.num == XMIT_ANTISPAM)
+		if (idp->val.status.mark == XMIT_ANTISPAM)
 		    errlen += strlen(idp->id) + 2;
 
 	    errmsg = alloca(errlen+3);
 	    (void) strcpy(errmsg, errhd);
 	    for (idp = xmit_names; idp; idp = idp->next)
-		if (idp->val.num == XMIT_ANTISPAM)
+		if (idp->val.status.mark == XMIT_ANTISPAM)
 		{
 		    strcat(errmsg, idp->id);
 		    if (idp->next)
@@ -1524,13 +1546,17 @@ int len;		/* length of message */
 flag forward;		/* TRUE to forward */
 {
     int	linelen;
-    char buf[MSGBUFSIZE+1];
+    unsigned char buf[MSGBUFSIZE+1];
+    unsigned char *inbufp = buf;
+    flag issoftline = FALSE;
 
     /* pass through the text lines */
     while (protocol->delimited || len > 0)
     {
-	if ((linelen = SockRead(sock, buf, sizeof(buf)-1)) == -1)
+	set_timeout(ctl->server.timeout);
+	if ((linelen = SockRead(sock, inbufp, sizeof(buf)-1-(inbufp-buf)))==-1)
 	{
+	    set_timeout(0);
 	    if (ctl->mda)
 	    {
 		if (sinkfp)
@@ -1539,7 +1565,7 @@ flag forward;		/* TRUE to forward */
 	    }
 	    return(PS_SOCKET);
 	}
-	set_timeout(ctl->server.timeout);
+	set_timeout(0);
 
 	/* write the message size dots */
 	if (linelen > 0)
@@ -1555,20 +1581,35 @@ flag forward;		/* TRUE to forward */
 	len -= linelen;
 
 	/* check for end of message */
-	if (protocol->delimited && *buf == '.')
-	    if (buf[1] == '\r' && buf[2] == '\n' && buf[3] == '\0')
+	if (protocol->delimited && *inbufp == '.')
+	    if (inbufp[1] == '\r' && inbufp[2] == '\n' && inbufp[3] == '\0')
 		break;
-	    else if (buf[1] == '\n' && buf[2] == '\0')
+	    else if (inbufp[1] == '\n' && inbufp[2] == '\0')
 		break;
 	    else
 		msglen--;	/* subtract the size of the dot escape */
 
 	msglen += linelen;
 
+	if (ctl->mimedecode && (mimemsg & MSG_NEEDS_DECODE)) {
+	    issoftline = UnMimeBodyline(&inbufp, (protocol->delimited && issoftline));
+	    if (issoftline && (sizeof(buf)-1-(inbufp-buf) < 200)) {
+		/* Soft linebreak, but less than 200 bytes left in
+		 * input buffer. Rather than doing a buffer overrun,
+		 * ignore the soft linebreak, NL-terminate data and
+		 * deliver what we have now.
+		 * (Who writes lines longer than 2K anyway?)
+		 */
+		*inbufp = '\n'; *(inbufp+1) = '\0';
+		issoftline = 0;
+	    }
+	}
+
 	/* ship out the text line */
-	if (forward)
+	if (forward && (!issoftline))
 	{
 	    int	n = stuffline(ctl, buf);
+	    inbufp = buf;
 
 	    if (n < 0)
 	    {
@@ -1752,7 +1793,7 @@ const struct method *proto;	/* protocol method table */
 
     /* set up the server-nonresponse timeout */
     sigsave = signal(SIGALRM, timeout_handler);
-    set_timeout(mytimeout = ctl->server.timeout);
+    mytimeout = ctl->server.timeout;
 
     if ((js = setjmp(restart)) == 1)
     {
@@ -1765,6 +1806,9 @@ const struct method *proto;	/* protocol method table */
 		  "timeout after %d seconds waiting for %s.",
 		  ctl->server.timeout,
 		  ctl->mda ? "MDA" : "SMTP");
+	else if (phase == LISTENER_WAIT)
+	    error(0, 0,
+		  "timeout after %d seconds waiting for listener to respond.");
 	else
 	    error(0, 0, "timeout after %d seconds.", ctl->server.timeout);
 
@@ -1779,7 +1823,7 @@ const struct method *proto;	/* protocol method table */
     else
     {
 	char buf [POPBUFSIZE+1], *realhost;
-	int *msgsizes, len, num, count, new, deletions = 0;
+	int *msgsizes, len, num, count, new, bytes, deletions = 0;
 #if INET6
 	int fetches, dispatches;
 #else /* INET6 */
@@ -1840,20 +1884,22 @@ const struct method *proto;	/* protocol method table */
 #ifdef KERBEROS_V4
 	if (ctl->server.preauthenticate == A_KERBEROS_V4)
 	{
+	    set_timeout(ctl->server.timeout);
 	    ok = kerberos_auth(sock, ctl->server.truename);
+	    set_timeout(0);
  	    if (ok != 0)
 		goto cleanUp;
-	    set_timeout(ctl->server.timeout);
 	}
 #endif /* KERBEROS_V4 */
 
 #ifdef KERBEROS_V5
 	if (ctl->server.preauthenticate == A_KERBEROS_V5)
 	{
+	    set_timeout(ctl->server.timeout);
 	    ok = kerberos5_auth(sock, ctl->server.truename);
+	    set_timeout(0);
  	    if (ok != 0)
 		goto cleanUp;
-	    set_timeout(ctl->server.timeout);
 	}
 #endif /* KERBEROS_V5 */
 
@@ -1861,14 +1907,16 @@ const struct method *proto;	/* protocol method table */
 	ok = (protocol->parse_response)(sock, buf);
 	if (ok != 0)
 	    goto cleanUp;
-	set_timeout(ctl->server.timeout);
 
 	/* try to get authorized to fetch mail */
 	if (protocol->getauth)
 	{
-	    shroud = ctl->password;
+	    if (protocol->password_canonify)
+		(protocol->password_canonify)(shroud, ctl->password);
+	    else
+		strcpy(shroud, ctl->password);
+
 	    ok = (protocol->getauth)(sock, ctl, buf);
-	    shroud = (char *)NULL;
 	    if (ok != 0)
 	    {
 		if (ok == PS_LOCKBUSY)
@@ -1885,7 +1933,6 @@ const struct method *proto;	/* protocol method table */
 		}
 		goto cleanUp;
 	    }
-	    set_timeout(ctl->server.timeout);
 	}
 
 	ctl->errcount = fetches = 0;
@@ -1905,28 +1952,59 @@ const struct method *proto;	/* protocol method table */
 			error(0, 0, "selecting or re-polling default folder");
 
 		/* compute # of messages and number of new messages waiting */
-		ok = (protocol->getrange)(sock, ctl, idp->id, &count, &new);
+		ok = (protocol->getrange)(sock, ctl, idp->id, &count, &new, &bytes);
 		if (ok != 0)
 		    goto cleanUp;
-		set_timeout(ctl->server.timeout);
+
+		/* 
+		 * We need the size of each message before it's
+		 * loaded in order to pass via the ESMTP SIZE
+		 * option.  If the protocol has a getsizes method,
+		 * we presume this means it doesn't get reliable
+		 * sizes from message fetch responses.
+		 */
+		if (count > 0 && proto->getsizes)
+		{
+		    int	i;
+
+		    msgsizes = (int *)alloca(sizeof(int) * count);
+		    for (i = 0; i < count; i++)
+			msgsizes[i] = -1;
+
+		    ok = (proto->getsizes)(sock, count, msgsizes);
+		    if (ok != 0)
+			goto cleanUp;
+
+		    if (bytes == -1)
+		    {
+			bytes = 0;
+			for (i = 0; i < count; i++)
+			    bytes += msgsizes[i];
+		    }
+		}
 
 		/* show user how many messages we downloaded */
 		if (idp->id)
 		    (void) sprintf(buf, "%s at %s (folder %s)",
 				   ctl->remotename, ctl->server.truename, idp->id);
 		else
-		    (void) sprintf(buf, "%s at %s", ctl->remotename, ctl->server.truename);
+		    (void) sprintf(buf, "%s at %s",
+				   ctl->remotename, ctl->server.truename);
 		if (outlevel > O_SILENT)
 		    if (count == -1)		/* only used for ETRN */
 			error(0, 0, "Polling %s", ctl->server.truename);
 		    else if (count != 0)
 		    {
 			if (new != -1 && (count - new) > 0)
-			    error(0, 0, "%d message%s (%d seen) for %s.",
+			    error_build("%d message%s (%d seen) for %s",
 				  count, count > 1 ? "s" : "", count-new, buf);
 			else
-			    error(0, 0, "%d message%s for %s.", 
+			    error_build("%d message%s for %s", 
 				  count, count > 1 ? "s" : "", buf);
+			if (bytes == -1)
+			    error_complete(0, 0, ".");
+			else
+			    error_complete(0, 0, " (%d bytes).", bytes);
 		    }
 		    else
 		    {
@@ -1976,27 +2054,6 @@ const struct method *proto;	/* protocol method table */
 		     */
 		    force_retrieval = !peek_capable && (ctl->errcount > 0);
 
-		    /* 
-		     * We need the size of each message before it's
-		     * loaded in order to pass via the ESMTP SIZE
-		     * option.  If the protocol has a getsizes method,
-		     * we presume this means it doesn't get reliable
-		     * sizes from message fetch responses.
-		     */
-		    if (proto->getsizes)
-		    {
-			int	i;
-
-			msgsizes = (int *)alloca(sizeof(int) * count);
-			for (i = 0; i < count; i++)
-			    msgsizes[i] = -1;
-
-			ok = (proto->getsizes)(sock, count, msgsizes);
-			if (ok != 0)
-			    goto cleanUp;
-			set_timeout(ctl->server.timeout);
-		    }
-
 		    /* read, forward, and delete messages */
 		    for (num = 1; num <= count; num++)
 		    {
@@ -2006,6 +2063,7 @@ const struct method *proto;	/* protocol method table */
 			    && (ctl->fetchall || force_retrieval || !(protocol->is_old && (protocol->is_old)(sock,ctl,num)));
 			flag suppress_delete = FALSE;
 			flag suppress_forward = FALSE;
+			flag suppress_readbody = FALSE;
 			flag retained = FALSE;
 
 			/*
@@ -2046,7 +2104,6 @@ const struct method *proto;	/* protocol method table */
 			    ok = (protocol->fetch_headers)(sock,ctl,num, &len);
 			    if (ok != 0)
 				goto cleanUp;
-			    set_timeout(ctl->server.timeout);
 
 			    /* -1 means we didn't see a size in the response */
 			    if (len == -1 && msgsizes)
@@ -2085,10 +2142,9 @@ const struct method *proto;	/* protocol method table */
 			    else if (ok == PS_REFUSED)
 				suppress_forward = TRUE;
 			    else if (ok == PS_TRUNCATED)
-				suppress_forward = TRUE;
+				suppress_readbody = TRUE;
 			    else if (ok)
 				goto cleanUp;
-			    set_timeout(ctl->server.timeout);
 
 			    /* 
 			     * If we're using IMAP4 or something else that
@@ -2098,14 +2154,13 @@ const struct method *proto;	/* protocol method table */
 			     * or other PS_REFUSED error response during
 			     * read_headers.
 			     */
-			    if (protocol->fetch_body) 
+			    if (protocol->fetch_body && !suppress_readbody) 
 			    {
 				if (outlevel == O_VERBOSE)
 				    fputc('\n', stderr);
 
 				if ((ok = (protocol->trail)(sock, ctl, num)))
 				    goto cleanUp;
-				set_timeout(ctl->server.timeout);
 				len = 0;
 				if (!suppress_forward)
 				{
@@ -2113,22 +2168,31 @@ const struct method *proto;	/* protocol method table */
 					goto cleanUp;
 				    if (outlevel > O_SILENT && !wholesize)
 					error_build(" (%d body bytes) ", len);
-				    set_timeout(ctl->server.timeout);
 				}
 			    }
 
 			    /* process the body now */
 			    if (len > 0)
 			    {
-				ok = readbody(sock,
-					      ctl,
-					      !suppress_forward,
-					      len);
-				if (ok == PS_TRANSIENT)
+			        if (suppress_readbody)
+				{
+				  /* When readheaders returns PS_TRUNCATED,
+				     the body (which has no content
+				     has already been read by readheaders,
+				     so we say readbody returned PS_SUCCESS */
+				  ok = PS_SUCCESS;
+				}
+				else
+				{
+				  ok = readbody(sock,
+					        ctl,
+					        !suppress_forward,
+					        len);
+				}
+			        if (ok == PS_TRANSIENT)
 				    suppress_delete = suppress_forward = TRUE;
 				else if (ok)
 				    goto cleanUp;
-				set_timeout(ctl->server.timeout);
 
 				/* tell server we got it OK and resynchronize */
 				if (protocol->trail)
@@ -2139,7 +2203,6 @@ const struct method *proto;	/* protocol method table */
 				    ok = (protocol->trail)(sock, ctl, num);
 				    if (ok != 0)
 					goto cleanUp;
-				    set_timeout(ctl->server.timeout);
 				}
 			    }
 
@@ -2226,8 +2289,8 @@ const struct method *proto;	/* protocol method table */
 			    struct idlist	*sdp;
 
 			    for (sdp = ctl->newsaved; sdp; sdp = sdp->next)
-				if (sdp->val.num == num)
-				    MARK_SEEN(sdp->val.num);
+				if (sdp->val.status.num == num)
+				    sdp->val.status.mark = UID_SEEN;
 			}
 
 			/* maybe we delete this message now? */
@@ -2246,7 +2309,6 @@ const struct method *proto;	/* protocol method table */
 			    ok = (protocol->delete)(sock, ctl, num);
 			    if (ok != 0)
 				goto cleanUp;
-			    set_timeout(ctl->server.timeout);
 #ifdef POP3_ENABLE
 			    delete_str(&ctl->newsaved, num);
 #endif /* POP3_ENABLE */
@@ -2269,7 +2331,6 @@ const struct method *proto;	/* protocol method table */
 	}
 
    no_error:
-	set_timeout(ctl->server.timeout);
 	ok = (protocol->logout_cmd)(sock, ctl);
 	/*
 	 * Hmmmm...arguably this would be incorrect if we had fetches but
@@ -2277,15 +2338,12 @@ const struct method *proto;	/* protocol method table */
 	 */
 	if (ok == 0)
 	    ok = (fetches > 0) ? PS_SUCCESS : PS_NOMAIL;
-	set_timeout(0);
 	close(sock);
 	goto closeUp;
 
     cleanUp:
-	set_timeout(ctl->server.timeout);
 	if (ok != 0 && ok != PS_SOCKET)
 	    (protocol->logout_cmd)(sock, ctl);
-	set_timeout(0);
 	close(sock);
     }
 
@@ -2406,13 +2464,16 @@ int size;	/* length of buffer */
     int oldphase = phase;	/* we don't have to be re-entrant */
 
     phase = SERVER_WAIT;
+    set_timeout(mytimeout);
     if (SockRead(sock, buf, size) == -1)
     {
+	set_timeout(0);
 	phase = oldphase;
 	return(PS_SOCKET);
     }
     else
     {
+	set_timeout(0);
 	if (buf[strlen(buf)-1] == '\n')
 	    buf[strlen(buf)-1] = '\0';
 	if (buf[strlen(buf)-1] == '\r')
@@ -2482,7 +2543,6 @@ va_dcl
 
     /* we presume this does its own response echoing */
     ok = (protocol->parse_response)(sock, buf);
-    set_timeout(mytimeout);
 
     phase = oldphase;
     return(ok);
