@@ -561,17 +561,12 @@ int main (int argc, char **argv)
 	     */
 	    {
 #ifndef __EMX__
-#ifndef SLEEP_WITH_ALARM	/* not normally on */
-                struct timeval timeout;
-
-                timeout.tv_sec = poll_interval;
-                timeout.tv_usec = 0;
-		lastsig = 0;
-                select(0,0,0,0, &timeout);
-#else
+#ifdef SLEEP_WITH_ALARM		/* not normally on */
 		/* 
-		 * This code is flaky under Red Hat Linux 5.0, and possibly
-		 * elsewhere.  John Stracke <francis@netscape.com> writes:
+		 * This code stopped working under glibc-2, apparently due
+		 * to the change in signal(2) semantics.  (The siginterrupt
+		 * line, added later, should fix this problem.) John Stracke
+		 * <francis@netscape.com> wrote:
 		 *
 		 * The problem seems to be that, after hitting the interval
 		 * timer while talking to the server, the process no longer
@@ -585,6 +580,22 @@ int main (int argc, char **argv)
 		 * to repoll--but, when the dummy server didn't respond, it
 		 * never timed out, and SIGALRM wouldn't make it.
 		 *
+		 * (continued below...)
+		 */
+		struct itimerval ntimeout;
+
+		ntimeout.it_interval.tv_sec = ntimeout.it_interval.tv_usec = 0;
+		ntimeout.it_value.tv_sec  = poll_interval;
+		ntimeout.it_value.tv_usec = 0;
+
+		siginterrupt(SIGALRM, 1);
+		setitimer(ITIMER_REAL,&ntimeout,NULL);
+		/* there's a small window here */
+		signal(SIGALRM, donothing);
+		pause();
+		signal(SIGALRM, SIG_IGN);
+#else
+		/* 
 		 * So the workaround I used is to make it sleep by using
 		 * select() instead of setitimer()/pause().  select() is
 		 * perfectly happy being called with a timeout and
@@ -594,16 +605,12 @@ int main (int argc, char **argv)
 		 * Unices where this is done, because select() is a library
 		 * function--but apparently not.
 		 */
-		struct itimerval ntimeout;
+                struct timeval timeout;
 
-		ntimeout.it_interval.tv_sec = ntimeout.it_interval.tv_usec = 0;
-		ntimeout.it_value.tv_sec  = poll_interval;
-		ntimeout.it_value.tv_usec = 0;
-
-		setitimer(ITIMER_REAL,&ntimeout,NULL);
-		signal(SIGALRM, donothing);
-		pause();
-		signal(SIGALRM, SIG_IGN);
+                timeout.tv_sec = poll_interval;
+                timeout.tv_usec = 0;
+		lastsig = 0;
+                select(0,0,0,0, &timeout);
 #endif
 #else /* EMX */
 		signal(SIGALRM, donothing);
@@ -687,6 +694,12 @@ static int load_params(int argc, char **argv, int optind)
 	    ctl = hostalloc(&cmd_opts);
 	    ctl->server.pollname = xstrdup(argv[optind]);
 
+	    /* avoid making pathological circular lists */
+	    ctl->server.localdomains = (struct idlist *)NULL;
+	    ctl->localnames = (struct idlist *)NULL;
+	    ctl->mailboxes = (struct idlist *)NULL;
+	    ctl->smtphunt = (struct idlist *)NULL;
+
 	foundit:
 	    ctl->active = TRUE;
 	}
@@ -714,7 +727,7 @@ static int load_params(int argc, char **argv, int optind)
 
 	    /* make sure we have a nonempty host list to forward to */
 	    if (!ctl->smtphunt)
-		save_str(&ctl->smtphunt, FALSE, fetchmailhost);
+		save_str(&ctl->smtphunt, fetchmailhost, FALSE);
 
 	    /* keep lusers from shooting themselves in the foot :-) */
 	    if (poll_interval && ctl->limit)
@@ -746,6 +759,7 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->forcecr, FALSE);
 	    DEFAULT(ctl->pass8bits, FALSE);
 	    DEFAULT(ctl->dropstatus, FALSE);
+	    DEFAULT(ctl->mimedecode, TRUE);
 	    DEFAULT(ctl->server.dns, TRUE);
 	    DEFAULT(ctl->server.uidl, FALSE);
 #undef DEFAULT
@@ -787,7 +801,7 @@ static int load_params(int argc, char **argv, int optind)
 
 	    /* if no folders were specified, set up the null one as default */
 	    if (!ctl->mailboxes)
-		save_str(&ctl->mailboxes, -1, (char *)NULL);
+		save_str(&ctl->mailboxes, (char *)NULL, 0);
 
 	    /* maybe user overrode timeout on command line? */
 	    if (ctl->server.timeout == -1)	
@@ -1097,10 +1111,10 @@ void dump_params (struct query *ctl)
 
 	printf("  Messages will be SMTP-forwarded to:");
 	for (idp = ctl->smtphunt; idp; idp = idp->next)
-	    if (ctl->server.protocol != P_ETRN || idp->val.num)
+	    if (ctl->server.protocol != P_ETRN || idp->val.status.mark)
 	    {
 		printf(" %s", idp->id);
-		if (!idp->val.num)
+		if (!idp->val.status.mark)
 	    	    printf(" (default)");
 	    }
 	printf("\n");
