@@ -30,13 +30,15 @@
  */
 #define		HOSTLEN		635	/* max hostname length (RFC1123) */
 #define		POPBUFSIZE	512	/* max length of response (RFC1939) */
-#define		USERNAMELEN	40	/* max POP3 arg length (RFC1939) */
 #define		IDLEN		128	/* max length of UID (RFC1939) */
 
+/* per RFC1939 this should be 40, but Microsoft Exchange ignores that limit */
+#define		USERNAMELEN	128	/* max POP3 arg length */
+
 /* clear a netBSD kernel parameter out of the way */ 
 #undef		MSGBUFSIZE
 
-#define		MSGBUFSIZE	2048   	/* size of message read buffer */
+#define		MSGBUFSIZE	998   	/* RFC822 limit on message line size */
 #define		PASSWORDLEN	64	/* max password length */
 #define		DIGESTLEN	33	/* length of MD5 digest */
 
@@ -66,21 +68,6 @@
 
 #define		SIZETICKER	1024	/* print 1 dot per this many bytes */
 
-/* we need to use zero as a flag-uninitialized value */
-#define FLAG_TRUE	2
-#define FLAG_FALSE	1
-
-struct idlist
-{
-    char *id;
-    union
-    {
-	int num;
-	char *id2;
-    } val;
-    struct idlist *next;
-};
-
 /*
  * We #ifdef this and use flag rather than bool
  * to avoid a type clash with curses.h
@@ -91,6 +78,30 @@ struct idlist
 #endif /* TRUE */
 typedef	char	flag;
 
+/* we need to use zero as a flag-uninitialized value */
+#define FLAG_TRUE	2
+#define FLAG_FALSE	1
+
+struct idlist
+{
+    char *id;
+    union
+    {
+	struct
+	{
+	    short	num;
+	    flag	mark;		/* UID-index information */
+#define UID_UNSEEN	0		/* hasn't been seen */
+#define UID_SEEN	1		/* seen, but not deleted */
+#define UID_DELETED	2		/* this message has been deleted */
+#define UID_EXPUNGED	3		/* this message has been expunged */ 
+        }
+	status;
+	char *id2;
+    } val;
+    struct idlist *next;
+};
+
 struct hostdata		/* shared among all user connections to given server */
 {
     /* rc file data */
@@ -158,6 +169,7 @@ struct query
     flag forcecr;		/* if TRUE, force CRs before LFs in text */
     flag pass8bits;		/* if TRUE, ignore Content-Transfer-Encoding */
     flag dropstatus;		/* if TRUE, drop Status lines in mail */
+    flag mimedecode;		/* if TRUE, decode MIME-coded headers/coded printable*/
     int	limit;			/* limit size of retrieved messages */
     int	fetchlimit;		/* max # msgs to get in single poll */
     int	batchlimit;		/* max # msgs to pass in single SMTP session */
@@ -174,17 +186,6 @@ struct query
     struct query *next;		/* next query control block in chain */
 };
 
-/*
- * UID-index information.  If the sign bit is on, this means the 
- * message UID has been seen or expunged and should be written
- * out to .fetchids at end of run.
- */
-#define UID_KEPT	0		/* remembered from a previous run */
-#define UID_DELETED	INT_MIN		/* this message has been deleted */
-#define UID_EXPUNGED	INT_MAX		/* this message has been expunged */ 
-#define SAVE_UID(n)	((n) <= 0)	/* must this UID be saved? */
-#define MARK_SEEN(n)	n *= -1		/* mark a UID seen */
-
 /*
  * Numeric option handling.  Numeric option value of zero actually means
  * it's unspecified.  Value less than zero is zero.
@@ -208,6 +209,7 @@ struct method
     flag tagged;		/* if true, generate & expect command tags */
     flag delimited;		/* if true, accept "." message delimiter */
     int (*parse_response)();	/* response_parsing function */
+    int (*password_canonify)();	/* canonicalize password */
     int (*getauth)();		/* authorization fetcher */
     int (*getrange)();		/* get message range to fetch */
     int (*getsizes)();		/* get sizes of messages */
@@ -299,7 +301,7 @@ char *nxtaddr(const char *);
 
 /* uid.c: UID support */
 void initialize_saved_lists(struct query *, const char *);
-struct idlist *save_str(struct idlist **, int, const char *);
+struct idlist *save_str(struct idlist **, const char *, flag);
 void free_str_list(struct idlist **);
 void save_str_pair(struct idlist **, const char *, const char *);
 void free_str_pair_list(struct idlist **);
@@ -307,6 +309,7 @@ int delete_str(struct idlist **, int);
 int str_in_list(struct idlist **, const char *);
 int str_nr_in_list(struct idlist **, const char *);
 int str_nr_last_in_list(struct idlist **, const char *);
+void str_set_mark( struct idlist **, const char *, const flag);
 int count_list( struct idlist **idl );
 char *str_from_nr_list( struct idlist **idl, int number );
 char *str_find(struct idlist **, int);
@@ -324,6 +327,15 @@ int prc_filecheck(const char *);
 void to64frombits(unsigned char *, const unsigned char *, int);
 int from64tobits(char *, const char *);
 
+/* unmime.c */
+/* Bit-mask returned by MimeBodyType */
+#define MSG_IS_7BIT       0x01
+#define MSG_IS_8BIT       0x02
+#define MSG_NEEDS_DECODE  0x80
+extern void UnMimeHeader(unsigned char *buf);
+extern int  MimeBodyType(unsigned char *hdrs);
+extern int  UnMimeBodyline(unsigned char **buf, int collapsedoubledot);
+
 /* interface.c */
 void interface_parse(char *, struct hostdata *);
 void interface_note_activity(struct hostdata *);
