@@ -55,10 +55,16 @@ int imap_ok(int sock, char *argbuf)
     seen = 0;
     do {
 	int	ok;
+	char	*cp;
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
 
+	/* all tokens in responses are caseblind */
+	for (cp = buf; *cp; cp++)
+	    if (islower(*cp))
+		*cp = toupper(*cp);
+
 	/* interpret untagged status responses */
 	if (strstr(buf, "* CAPABILITY"))
 	    strncpy(capabilities, buf + 12, sizeof(capabilities));
@@ -81,7 +87,7 @@ int imap_ok(int sock, char *argbuf)
 	    unseen = atoi(cp);
 	}
 	if (strstr(buf, "FLAGS"))
-	    seen = (strstr(buf, "Seen") != (char *)NULL);
+	    seen = (strstr(buf, "SEEN") != (char *)NULL);
     } while
 	(tag[0] != '\0' && strncmp(buf, tag, strlen(tag)));
 
@@ -562,18 +568,13 @@ static int do_gssauth(int sock, char *hostname, char *username)
 }	
 #endif /* GSSAPI */
 
-static char *canonify_imap_password(char *passwd)
+int imap_canonicalize(char *result, char *passwd)
 /* encode an IMAP password as per RFC1730's quoting conventions */
 {
-    char *result;
     int i, j;
 
-    result = malloc(2*strlen(passwd));
-    if (!result)
-	return 0;
-
-    j=0;
-    for (i=0; i<strlen(passwd); ++i)
+    j = 0;
+    for (i = 0; i < strlen(passwd); i++)
     {
 	if ((passwd[i] == '\\') || (passwd[i] == '"'))
 	    result[j++] = '\\';
@@ -581,20 +582,21 @@ static char *canonify_imap_password(char *passwd)
     }
     result[j] = '\0';
 
-    return(result);
+    return(i);
 }
 
 int imap_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
     int ok = 0;
+    char	password[PASSWORDLEN*2];
 
     /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
     capabilities[0] = '\0';
     if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
     {
 	/* UW-IMAP server 10.173 notifies in all caps */
-	if (strstr(capabilities, "IMAP4rev1") || strstr(capabilities, "IMAP4REV1"))
+	if (strstr(capabilities, "IMAP4REV1"))
 	{
 	    imap_version = IMAP4rev1;
 	    if (outlevel == O_VERBOSE)
@@ -670,26 +672,18 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     }
 #endif /* KERBEROS_V4 */
 
+#ifdef __UNUSED__	/* The Cyrus IMAP4rev1 server chokes on this */
     /* this handles either AUTH=LOGIN or AUTH-LOGIN */
     if ((imap_version >= IMAP4rev1) && (!strstr(capabilities, "LOGIN"))) {
       error(0,-1, "Required LOGIN capability not supported by server");
       return PS_AUTHFAIL;
     };
+#endif /* __UNUSED__ */
 
-    /* try to get authorized in the ordinary (AUTH=LOGIN) way */
-    {
-       char *newpass = canonify_imap_password(ctl->password);
-       
-       if (!newpass)
-          return(PS_AUTHFAIL); /* should report error better!!!! */
-       
-       ok = gen_transact(sock, "LOGIN %s \"%s\"", ctl->remotename, newpass);
-       
-       free(newpass);
-    
-       if (ok)
-          return(ok);
-    }
+    imap_canonicalize(password, ctl->password);
+    ok = gen_transact(sock, "LOGIN \"%s\" \"%s\"", ctl->remotename, password);
+    if (ok)
+	return(ok);
     
     return(PS_SUCCESS);
 }
@@ -708,18 +702,20 @@ static int internal_expunge(int sock)
 #ifdef IMAP_UID	/* not used */
     expunge_uids(ctl);
 #endif /* IMAP_UID */
+
+    return(PS_SUCCESS);
 }
 
 static int imap_getrange(int sock, 
 			 struct query *ctl, 
 			 const char *folder, 
-			 int *countp, int *newp)
+			 int *countp, int *newp, int *bytes)
 /* get range of messages to be fetched */
 {
     int ok;
 
     /* find out how many messages are waiting */
-    recent = unseen = -1;
+    *bytes = recent = unseen = -1;
 
     if (pass > 1)
     {
@@ -986,6 +982,7 @@ const static struct method imap =
     TRUE,		/* this is a tagged protocol */
     FALSE,		/* no message delimiter */
     imap_ok,		/* parse command response */
+    imap_canonicalize,	/* deal with embedded slashes and spaces */
     imap_getauth,	/* get authorization */
     imap_getrange,	/* query range of messages */
     imap_getsizes,	/* get sizes of messages (used for --limit option */
