@@ -133,6 +133,9 @@ parse_netrc (file)
 	/* If we are defining macros, then skip parsing the line. */
 	while (*p && last_token != tok_macdef)
 	{
+	    char quote_char = 0;
+	    char *pp;
+
 	    /* Skip any whitespace. */
 	    while (*p && isspace (*p))
 		p++;
@@ -141,15 +144,41 @@ parse_netrc (file)
 	    if (*p == '#')
 		break;
 
-	    tok = p;
+	    tok = pp = p;
 
 	    /* Find the end of the token. */
-	    while (*p && !isspace (*p))
-		p ++;
-
+	    while (*p && (quote_char || !isspace (*p)))
+	    {
+		if (quote_char)
+		{
+		    if (quote_char == *p)
+		    {
+			quote_char = 0;
+			p ++;
+		    }
+		    else
+		    {
+			*pp = *p;
+			p ++;
+			pp ++;
+		    }
+		}
+		else
+		{
+		    if (*p == '"' || *p == '\'')
+			quote_char = *p;
+		    else
+		    {
+			*pp = *p;
+			pp ++;
+		    }
+		    p ++;
+		}
+	    }
 	    /* Null-terminate the token, if it isn't already. */
 	    if (*p)
 		*p ++ = '\0';
+	    *pp = 0;
 
 	    switch (last_token)
 	    {
@@ -210,9 +239,6 @@ parse_netrc (file)
 	    else
 	    {
 		/* Fetch the next token. */
-		if (!strcmp (tok, "account"))
-		    last_token = tok_account;
-
 		if (!strcmp (tok, "default"))
 		{
 		    maybe_add_to_list (&current, &retval);
@@ -232,6 +258,9 @@ parse_netrc (file)
 		else if (!strcmp (tok, "password"))
 		    last_token = tok_password;
 
+		else if (!strcmp (tok, "account"))
+		    last_token = tok_account;
+
 		else
 		{
 		    fprintf (stderr, "%s:%d: warning: unknown token \"%s\"\n",
