@@ -56,9 +56,9 @@
 #define LA_MONITOR      38
 #define LA_YYDEBUG	39
 
-/* options still left: CgGhHjJoORUwWxXYzZ */
+/* options still left: CgGhHjJoORUwWxXYz */
 static const char *shortoptions = 
-	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:b:B:e:m:I:M:y";
+	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:Z:b:B:e:m:T:I:M:y";
 
 static const struct option longoptions[] = {
 /* this can be const because all flag fields are 0 and will never get set */
@@ -312,7 +312,7 @@ struct query *ctl;	/* option record to be initialized */
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
-		save_str(&ctl->mailboxes, -1, cp);
+		save_str(&ctl->mailboxes, cp, 0);
 	    } while
 		((cp = strtok((char *)NULL, ",")));
 	    break;
@@ -321,7 +321,7 @@ struct query *ctl;	/* option record to be initialized */
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
-		save_str(&ctl->smtphunt, TRUE, cp);
+		save_str(&ctl->smtphunt, cp, TRUE);
 	    } while
 		((cp = strtok((char *)NULL, ",")));
 	    ocount++;
