@@ -117,7 +117,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 #ifdef RPA_ENABLE
 	 /*
 	  * CompuServe has changed its RPA behavior.  Used to be they didn't
-	  * accept PASS, but I'm told this changed in mid-November.
+	  * accept PASS, but I'm told this changed in mid-November 1997.
 	  */
 	 if (strstr(greeting, "csi.com"))
 	 {
@@ -216,14 +216,13 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	ok = PS_ERROR;
     }
 
-    /* maybe we detected a lock-busy condition? */
     if (ok != 0)
     {
+	/* maybe we detected a lock-busy condition? */
         if (ok == PS_LOCKBUSY)
-	{
 	    error(0, 0, "lock busy!  Is another session active?"); 
-	    return(PS_LOCKBUSY);
-	}
+
+	return(ok);
     }
 
     /*
@@ -332,11 +331,14 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 	    }
 	} 
     }
-    /* The first try_id messages are known -> copy them to
-       the newsaved list */
+    /* the first try_id messages are known -> copy them to the newsaved list */
     for( num = first_nr; num < list_len; num++ )
-	save_str(&ctl->newsaved, num-first_nr + 1,
-		 str_from_nr_list( &ctl->oldsaved, num ));
+    {
+	struct idlist	*new = save_str(&ctl->newsaved, 
+				str_from_nr_list(&ctl->oldsaved, num),
+				UID_UNSEEN);
+	new->val.status.num = num - first_nr + 1;
+    }
 
     if( nolinear ) {
 	free_str_list(&ctl->oldsaved);
@@ -351,7 +353,7 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 static int pop3_getrange(int sock, 
 			 struct query *ctl,
 			 const char *folder,
-			 int *countp, int *newp)
+			 int *countp, int *newp, int *bytes)
 /* get range of messages to be fetched */
 {
     int ok;
@@ -373,7 +375,7 @@ static int pop3_getrange(int sock,
     gen_send(sock, "STAT");
     ok = pop3_ok(sock, buf);
     if (ok == 0)
-	sscanf(buf,"%d %*d", countp);
+	sscanf(buf,"%d %d", countp, bytes);
     else
 	return(ok);
 
@@ -425,10 +427,17 @@ static int pop3_getrange(int sock,
  			break;
  		    else if (sscanf(buf, "%d %s", &num, id) == 2)
 		    {
- 			save_str(&ctl->newsaved, num, id);
+ 			struct idlist	*new;
+
+			new = save_str(&ctl->newsaved, id, UID_UNSEEN);
+			new->val.status.num = num;
 
 			/* note: ID comparison is caseblind */
-			if (!str_in_list(&ctl->oldsaved, id))
+			if (str_in_list(&ctl->oldsaved, id)) {
+			    new->val.status.mark = UID_SEEN;
+			    str_set_mark(&ctl->oldsaved, id, UID_SEEN);
+			}
+			else
 			    (*newp)++;
 		    }
  		}
@@ -436,7 +445,7 @@ static int pop3_getrange(int sock,
  	}
     }
 
-    return(0);
+    return(PS_SUCCESS);
 }
 
 static int pop3_getsizes(int sock, int count, int *sizes)
@@ -526,6 +535,7 @@ const static struct method pop3 =
     FALSE,		/* this is not a tagged protocol */
     TRUE,		/* this uses a message delimiter */
     pop3_ok,		/* parse command response */
+    NULL,		/* no password canonicalization */
     pop3_getauth,	/* get authorization */
     pop3_getrange,	/* query range of messages */
     pop3_getsizes,	/* we can get a list of sizes */
