@@ -8,7 +8,11 @@
 #define SOCKET__
 
 /* Create a new client socket; returns (FILE *)NULL on error */
-int SockOpen(char *host, int clientPort);
+#if INET6
+int SockOpen(const char *host, const char *service, const char *options);
+#else /* INET6 */
+int SockOpen(const char *host, int clientPort, const char *options);
+#endif /* INET6 */
 
 /* 
 Get a string terminated by an '\n' (matches interface of fgets).
