@@ -4,7 +4,7 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
